﻿--[[
-- Scale Everything is written by Macniel
-- 
--]]

-- Saved Variables
SCALETABLE = {
 ["PlayerFrame"] = 1.0,
 ["Minimap"] = 1.0,
 ["TargetFrame"] = 1.0,
 ["MainMenuBar"] = 0.75,
}

POINTTABLE = {
}

-- Default Variables

local DEFAULT_SCALETABLE = {}
local DEFAULT_POINTTABLE = {}

-- AddOn Lua is now following

ScaleEverything = CreateFrame("Frame", "ScaleEverything")

-- Interfacepage creation and widgets

local setupPanel = CreateFrame("Frame")

local iter = 0

local findInTable = function(t, name)
	local i = 0
	for k,v in pairs(t) do
		i = i +1
		if name == k then
			return i
		end
	end
	return -1
end

function ScaleEverything:CreateEraseButton(parent, frame)
	local btn = CreateFrame("Button", "RemoveButton"..frame, parent)
	btn:SetWidth(16)
	btn:SetHeight(16)
	btn.frame = frame
	btn:SetDisabledTexture("interface\\buttons\\ui-minusbutton-disabled")
	btn:SetPushedTexture("interface\\buttons\\ui-minusbutton-down")
	btn:SetHighlightTexture("interface\\buttons\\ui-plusbutton-hilight")
	btn:SetNormalTexture("interface\\buttons\\ui-minusbutton-up")
	btn:SetScript("OnEnter", function(btn)
		GameTooltip:SetOwner(btn, "ANCHOR_RIGHT")
		GameTooltip:SetText(string.format(SE_GetLocaleString("TOOLTIP_REMOVE"), btn.frame))
		GameTooltip:Show()
	end)
	btn:SetScript("OnLeave", function(btn)
		GameTooltip:ClearLines()
		GameTooltip:Hide()
	end)
	btn:SetScript("OnClick", function(btn, button)
		if DEFAULT_POINTTABLE[btn.frame] then
			getglobal(btn.frame):ClearAllPoints()
			getglobal(btn.frame):SetPoint(unpack(DEFAULT_POINTTABLE[btn.frame]))
			
			local index = findInTable(DEFAULT_POINTTABLE, btn.frame)
			
			DEFAULT_POINTTABLE[btn.frame] = nil
		end
		if POINTTABLE[btn.frame] then
			local index = findInTable(POINTTABLE, btn.frame)
			
			POINTTABLE[btn.frame] = nil
		end
		if DEFAULT_SCALETABLE[btn.frame] then
			getglobal(btn.frame):SetScale(DEFAULT_SCALETABLE[btn.frame])
			
			local index = findInTable(DEFAULT_SCALETABLE, btn.frame)
			DEFAULT_SCALETABLE[btn.frame] = "NA"
		end
		if SCALETABLE[btn.frame] then

			local index = findInTable(SCALETABLE, btn.frame)
			SCALETABLE[btn.frame] = "NA"
		end
		ScaleEverything:CreatePage(true)
	end)
	
	return btn
end

function ScaleEverything:CreateLocker(parent, frame) -- create a checkbox to lock/unlock a frame
	local checkbox = CreateFrame("CheckButton", "UnlockButton"..frame, parent, "InterfaceOptionsCheckButtonTemplate")
	local text = getglobal("UnlockButton"..frame.."Text")
	text:SetText(SE_GetLocaleString("ENABLE_DRAG"))
	text:SetWidth(80)
	text:SetJustifyH("LEFT")
	text:SetFontObject("GameFontHighlightSmall")
	checkbox.frame = frame
	checkbox:RegisterForClicks("LeftButtonUp", "RightButtonUp")
	
	checkbox:SetScript("OnClick", function(checkbox, button)
		local affected = getglobal(checkbox.frame)
		if button == "LeftButton" then
			if checkbox:GetChecked() then
				if not DEFAULT_POINTTABLE[checkbox.frame] then
					DEFAULT_POINTTABLE[checkbox.frame] = {getglobal(checkbox.frame):GetPoint()}
				end
				
				checkbox.orgBackground = affected:GetBackdrop()
				checkbox.orgBackgroundColor = affected:GetBackdropColor()
				affected:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background"})
				affected:SetBackdropColor(1,0,0,0.5)
				
				if not affected:IsMouseEnabled() then
					checkbox.mouseState = 1
					affected:EnableMouse(true)
				end
				affected:SetMovable(true)
				affected:RegisterForDrag("RightButton")
				checkbox.orgDragStart = affected:GetScript("OnDragStart")
				checkbox.orgDragStop = affected:GetScript("OnDragStop")
				affected:SetScript("OnDragStart", function(self)
					self:ClearAllPoints()
					self:StartMoving()
				end)
				affected:SetScript("OnDragStop", function(self)
					self:StopMovingOrSizing()
					POINTTABLE[checkbox.frame] = {self:GetPoint()}
					
					if type(POINTTABLE[checkbox.frame][2]) == "table" then
						POINTTABLE[checkbox.frame][2] = POINTTABLE[checkbox.frame][2]:GetName()
					end
				end)
			else
				affected:SetScript("OnDragStart", self.orgDragStart)
				affected:SetScript("OnDragStop", self.orgDragStop)
				if checkbox.mouseState then
					affected:EnableMouse(false)
				end
				affected:SetBackdrop(checkbox.orgBackground)
				affected:SetBackdropColor(checkbox.orgBackgroundColor)
			end
		else
			checkbox:SetChecked(false)
			if checkbox.mouseState then
				affected:EnableMouse(false)
			end
			affected:SetBackdrop(checkbox.orgBackground)
			affected:SetBackdropColor(checkbox.orgBackgroundColor)
			if type(DEFAULT_POINTTABLE[checkbox.frame]) == "table" then
				affected:ClearAllPoints()
				affected:orgPoint(unpack(DEFAULT_POINTTABLE[checkbox.frame]))
				affected:SetScript("OnDragStart", self.orgDragStart)
				affected:SetScript("OnDragStop", self.orgDragStop)
				POINTTABLE[checkbox.frame] = {affected:GetPoint()}
				
				if type(POINTTABLE[checkbox.frame][2]) == "table" then
					POINTTABLE[checkbox.frame][2] = POINTTABLE[checkbox.frame][2]:GetName()
				end
			end
		end
	end)
	checkbox:SetScript("OnEnter", function(self)
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
		if self:GetChecked() then
			GameTooltip:SetText(string.format(SE_GetLocaleString("TOOLTIP_DRAG"), self.frame))
		else
			GameTooltip:SetText(string.format(SE_GetLocaleString("TOOLTIP_NODRAG"), self.frame))
		end
		GameTooltip:Show()
	end)
	checkbox:SetScript("OnLeave", function(self)
		GameTooltip:Hide()
	end)
	
	return checkbox
end

function ScaleEverything:CreateSlider(parent, frame, blank) -- create a slider to change scaling
	local slider = CreateFrame("Slider", "ScaleSlider"..frame, parent, "OptionsSliderTemplate")
		
		slider:SetMinMaxValues(0.1, 2)
		slider:SetWidth(92)
		slider.frame = frame
		slider.caption = getglobal("ScaleSlider"..frame.."Text")
		slider.caption:SetParent(parent)
		slider.caption:ClearAllPoints()
		slider.caption:SetFontObject("GameFontHighlightSmall")
		slider.caption:SetPoint("RIGHT", slider, "LEFT", -16, 0)
		slider.caption:SetWidth(110)
		slider.caption:SetJustifyH("LEFT")
		slider.caption:SetText(frame)
		if not blank then
			slider:EnableMouseWheel(true)
			if SCALETABLE[frame] ~= "NA" then
				slider:SetValue(getglobal(frame):GetScale())
				getglobal("ScaleSlider"..frame.."Low"):SetFormattedText("%.2f",slider:GetMinMaxValues())
				getglobal("ScaleSlider"..frame.."High"):SetFormattedText("%.2f",select(2,slider:GetMinMaxValues()))
				
				slider:SetScript("OnMouseWheel", function(self, arg1)
					slider:SetValue(slider:GetValue() +( arg1 / 100))	
				end)
				slider:SetScript("OnShow", function(self)
					self:SetValue(SCALETABLE[self.frame])
					GameTooltip:Hide()
				end)
				slider:SetScript("OnValueChanged", function(self)
					SCALETABLE[self.frame] = self:GetValue()
					GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
					GameTooltip:SetText(string.format(SE_GetLocaleString("TOOLTIP_SCALE"), self.frame, SCALETABLE[self.frame]*100))
					GameTooltip:Show()
					if getglobal(self.frame).orgScale then
						getglobal(self.frame):orgScale(self:GetValue())
						self.caption:SetVertexColor(1,1,1)
					else
						getglobal(self.frame):SetScale(self:GetValue())
						self.caption:SetVertexColor(1,0,0)
					end
				end)	slider:SetScript("OnEnter", function(self)
					GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
					GameTooltip:SetText(string.format(SE_GetLocaleString("TOOLTIP_SCALE"), self.frame, SCALETABLE[self.frame]*100))
					GameTooltip:Show()
				end)
				slider:SetScript("OnLeave", function(self)
					GameTooltip:ClearLines()
					GameTooltip:Hide()
				end)
			else
				slider:SetScript("OnValueChanged", function() end)
				slider:SetValue(0)
				--slider:Hide()
				--getglobal("UnlockButton"..frame):Hide()
			end
		end
	return slider
end

function ScaleEverything:CreatePage(refreshOnly) -- Interfacepage
	if not refreshOnly then
		setupPanel.name = SE_GetLocaleString("PAGE_NAME")
		local SE_title = setupPanel:CreateFontString() 
		SE_title:SetFontObject(GameFontNormalLarge)
		SE_title:SetPoint("TOPLEFT", 16, -16)
		SE_title:SetText(SE_GetLocaleString("PAGE_NAME"))
 
		local SE_subText = setupPanel:CreateFontString()
		SE_subText:SetFontObject(GameFontHighlightSmall)
		SE_subText:SetPoint("TOPLEFT", SE_title, "BOTTOMLEFT", 0, -8)
		SE_subText:SetPoint("RIGHT", -30, 0)
		SE_subText:SetHeight(64)
		SE_subText:SetJustifyH("LEFT")
		SE_subText:SetJustifyV("TOP")
		
		SE_subText:SetText(SE_GetLocaleString("PAGE_DESCRIPTION"))  
	
		
	
		local ScrollFrame = CreateFrame("ScrollFrame", "SE_ScrollFrame", setupPanel, "UIPanelScrollFrameTemplate")
		
		local background = CreateFrame("Frame", nil, ScrollFrame)
		
		background:SetPoint("TOPLEFT", "SE_ScrollFrameScrollBarScrollUpButton", -6, 6)
		background:SetPoint("BOTTOMRIGHT", "SE_ScrollFrameScrollBarScrollDownButton", 6, -6)

		background:SetBackdrop({bgFile = nil, 
                                            edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", 
                                            tile = true, tileSize = 16, edgeSize = 16, 
                                            insets = { left = 4, right = 4, top = 4, bottom = 4 }});

		
		ScrollFrame:SetPoint("TOPLEFT", SE_subText, "BOTTOMLEFT", -16, -8)
		ScrollFrame:SetPoint("BOTTOMLEFT", setupPanel, "BOTTOMLEFT", 0, 10)
		ScrollFrame:SetPoint("RIGHT", setupPanel, "RIGHT", -34, 0)
	
		local ScrollChild = CreateFrame("Frame", "SE_ScrollChild", SE_ScrollFrame)
	
		
		ScrollChild:SetPoint("TOPLEFT")
		ScrollChild:SetWidth(299)
	
		ScrollFrame:SetScrollChild(ScrollChild)
		
		InterfaceOptions_AddCategory(setupPanel)  
		
	end
	
	for i, children in pairs({SE_ScrollChild:GetChildren()}) do
		children:Hide()
	end
	iter = 0
	for frame, value in pairs(SCALETABLE) do
		
			slider = getglobal("ScaleSlider"..frame) or ScaleEverything:CreateSlider(SE_ScrollChild, frame)
			slider:SetPoint("TOP", 40, -(iter * (slider:GetHeight() + 16)))
			checkbox = getglobal("UnlockButton"..frame) or ScaleEverything:CreateLocker(SE_ScrollChild, frame)
			checkbox:SetPoint("LEFT", slider, "RIGHT", 16,0)
			removeButton = getglobal("RemoveButton"..frame) or ScaleEverything:CreateEraseButton(SE_ScrollChild, frame)
			removeButton:SetPoint("LEFT", getglobal("UnlockButton"..frame.."Text"), "RIGHT", 0, 0)			
			removeButton:SetFrameLevel(checkbox:GetFrameLevel() + 2)
			iter = iter + 1
			SE_ScrollChild:SetHeight(SE_ScrollChild:GetHeight() + slider:GetHeight() + 16)
			slider:Show()
			checkbox:Show()
			removeButton:Show()
	end
	
	setupPanel.default = function()
		for frame, value in pairs(DEFAULT_SCALETABLE) do
			getglobal(frame):orgScale(value)
			getglobal(frame):orgPoint(DEFAULT_POINTTABLE[frame])
		end
	end
	
	setupPanel.cancel = function()
		for frame, value in pairs(SCALETABLE) do
			value = getglobal(frame):GetScale()
		end
	end
	
	setupPanel.okay = function()
		for frame, value in pairs(SCALETABLE) do
			getglobal(frame):orgScale(value)
			SCALETABLE[frame] = value
		end
	end
	
	
end

setupPanel:SetScript("OnShow", function() -- Refresh  Interfacepage
	ScaleEverything:CreatePage(true)
end)

ScaleEverything:SetScript("OnEvent", function(self, event) -- load Variables and store default attributes
	if event == "PLAYER_ENTERING_WORLD" then
		for frame, value in pairs(SCALETABLE) do
			if getglobal(frame) and getglobal(frame).SetScale and value ~= "NA" then
				getglobal(frame):SetScale(value)
			end
			if type(POINTTABLE[frame]) == "table" then
				getglobal(frame):ClearAllPoints()
				getglobal(frame):SetPoint(unpack(POINTTABLE[frame]))
				getglobal(frame).orgPoint = getglobal(frame).SetPoint
				getglobal(frame).orgScale = getglobal(frame).SetScale
				getglobal(frame).SetPoint = function() end
				getglobal(frame).SetScale = function() end
				
			end
		end
		ScaleEverything:CreatePage()
		ScaleEverything:CreateAdvancedPage()
	end
end)

ScaleEverything:RegisterEvent("VARIABLES_LOADED")
ScaleEverything:RegisterEvent("PLAYER_ENTERING_WORLD")

SlashCmdList["SCALEEVERYTHING"] = function(cmd) -- frame-adding support via slash command
	if cmd == "-gui" then
		InterfaceOptionsFrame_OpenToCategory(setupPanel)

	elseif cmd == "-hover" then
		if GetMouseFocus():GetName() then
			SCALETABLE[GetMouseFocus():GetName()] = GetMouseFocus():GetScale() or 1.0
			DEFAULT_CHAT_FRAME:AddMessage(string.format(SE_GetLocaleString("SUCCESS"), GetMouseFocus():GetName()))
			
			GetMouseFocus().orgPoint = GetMouseFocus().SetPoint
			GetMouseFocus().orgScale = GetMouseFocus().SetScale
			GetMouseFocus().SetPoint = function() end
			GetMouseFocus().SetScale = function() end
		end
	elseif cmd ~= "" then
		local func, err = loadstring("return " ..cmd)
		if type(func()) == "string" then
			cmd = func()
		end
		if type(SCALETABLE) == "table" and getglobal(cmd).GetScale then
			SCALETABLE[cmd] = getglobal(cmd):GetScale()
			DEFAULT_CHAT_FRAME:AddMessage(string.format(SE_GetLocaleString("SUCCESS"), cmd))
		else
			SCALETABLE = {}
			if type(cmd) == "table" and getglobal(cmd).GetScale then
				SCALETABLE[cmd] = getglobal(cmd):GetScale()
				DEFAULT_CHAT_FRAME:AddMessage(string.format(SE_GetLocaleString("SUCCESS"), cmd))
			else
				SCALETABLE[cmd] = "NA"
				DEFAULT_CHAT_FRAME:AddMessage(string.format(SE_GetLocaleString("FAILED"), cmd))
			end
		end
		getglobal(cmd).orgPoint = getglobal(cmd).SetPoint
		getglobal(cmd).orgScale = getglobal(cmd).SetScale
		getglobal(cmd).SetPoint = function() end
		getglobal(cmd).SetScale = function() end
	end
end
SLASH_SCALEEVERYTHING1 = "/se"

